# coding: utf-8
u"""Вспомогательные средства."""


def is_ranges_intersected(range1, range2):
    u"""Возвращает True, если указанные интервалы значений пересекаются.

    Интервалы задаются в виде двухэлементных кортежей, первый элемент кортежа
    определяет начало интервала, а второй - конец интервала. None определяет
    открытый с соответствующей стороны интервал.

    Типы данных в интервалах должны поддерживать сравнение значений с помощью
    оператора <=.

    :rtype: bool
    """
    (from1, to1), (from2, to2) = range1, range2

    assert from1 is None or to1 is None or from1 <= to1, (from1, to1)
    assert from2 is None or to2 is None or from2 <= to2, (from2, to2)

    if from1 is None and to1 is None:
        result = True

    elif from1 is not None and to1 is None:
        result = to2 is None or from1 <= to2

    elif from1 is None and to1 is not None:
        result = from2 is None or from2 <= to1

    else:  # from1 is not None and to1 is not None
        if from2 is None and to2 is None:
            result = True

        elif from2 is not None and to2 is None:
            result = from2 <= to1

        elif from2 is None and to2 is not None:
            result = from1 <= to2

        else:  # from2 is not None and to2 is not None
            result = from2 <= to1 and from1 <= to2

    return result
